( cd usr/lib64 ; rm -rf libqca.so )
( cd usr/lib64 ; ln -sf libqca.so.1.0.0 libqca.so )
( cd usr/lib64 ; rm -rf libqca.so.1 )
( cd usr/lib64 ; ln -sf libqca.so.1.0.0 libqca.so.1 )
( cd usr/lib64 ; rm -rf libqca.so.1.0 )
( cd usr/lib64 ; ln -sf libqca.so.1.0.0 libqca.so.1.0 )
